use std::collections::HashMap;
use base::read_file;

#[derive(Debug, Clone)]
struct CrateStorage {
    stacks: HashMap<usize, Vec<char>>
}

impl CrateStorage {
    fn print_top(&self) {
        for k in 0..(9 as usize) {
            match self.stacks.get(&k) {
                None => {
                    print!(" ")
                }
                Some(v) => {
                    print!("{}", v[v.len() - 1]);
                }
            }
        }
        println!()
    }
}

#[derive(Debug)]
struct MoveOp {
    amount: usize,
    from: usize,
    to: usize
}

impl MoveOp {
    fn apply(&self, storge: &mut CrateStorage, task_1 : bool) {
        if task_1 {
            for _ in 0..self.amount {
                let vec = storge.stacks.get_mut(&self.from).unwrap();
                let element = vec.remove(vec.len() - 1);
                storge.stacks.get_mut(&self.to).unwrap().push(element);
            }
        } else {
            let mut temp = vec![];
            for _ in 0..self.amount {
                let vec = storge.stacks.get_mut(&self.from).unwrap();
                temp.push(vec.remove(vec.len() - 1));
            }
            temp.reverse();
            for i in 0..self.amount {
                storge.stacks.get_mut(&self.to).unwrap().push(temp[i]);
            }
        }
        dbg!(&storge);
    }
}

fn main() {
    let lines = read_file("day5_test.txt");
    let mut storage = CrateStorage{
        stacks: HashMap::new()
    };

    let mut moves =vec![];

    for line in lines {
        if line.starts_with("[") {
            let mut iter = line.chars().enumerate();
            loop {
                match iter.next() {
                    Some((idx, char)) => {
                        if char == '[' {
                            let idx = idx / 4;
                            if let Some((_, value)) = iter.next() {
                                let vec = storage.stacks.entry(idx).or_insert_with(|| Vec::new());
                                vec.insert(0, value);
                            } else {
                                break;
                            }
                        }
                    }
                    None => break
                }
            }
        } else if line.starts_with("move") {
            let split = line.split_whitespace().collect::<Vec<_>>();
            let amount = split[1].parse::<usize>().unwrap();
            let from = split[3].parse::<usize>().unwrap() - 1;
            let to = split[5].parse::<usize>().unwrap() - 1;
            moves.push(MoveOp{amount, from, to});
        }
    }

    let mut storage_task_1 = storage.clone();
    moves.iter().for_each(|move_op| move_op.apply(&mut storage_task_1, true));
    storage_task_1.print_top();

    let mut storage_task_2 = storage;
    moves.iter().for_each(|move_op| move_op.apply(&mut storage_task_2, false));
    storage_task_2.print_top();


}